
#ifndef RZ_IL_OPBUILDER_BEGIN_H
#error rz_il_opbuilder_end.h included without rz_il_opbuilder_begin.h before
#endif

#undef ITE

#undef UN
#undef U8
#undef U16
#undef U32
#undef U64

#undef SN
#undef S8
#undef S16
#undef S32
#undef S64

#undef IL_FALSE
#undef IL_TRUE
#undef INV
#undef XOR
#undef AND

#undef ADD
#undef SUB
#undef MUL
#undef DIV
#undef SDIV
#undef SHIFTL
#undef SHIFTR
#undef SHIFTL0
#undef SHIFTR0
#undef SHIFTRA
#undef LOGAND
#undef LOGOR
#undef LOGXOR
#undef LOGNOT
#undef NEG

#undef NON_ZERO
#undef IS_ZERO
#undef MSB
#undef LSB
#undef EQ
#undef UGT
#undef ULT
#undef ULE
#undef UGT
#undef UGE
#undef SLT
#undef SLE
#undef SGT
#undef SGE

#undef LOAD
#undef LOADW
#undef STORE
#undef STOREW

#undef VARG
#undef VARL
#undef VARLP
#undef SETG
#undef SETL
#undef LET

#undef SEQ2
#undef SEQ3
#undef SEQ4
#undef SEQ5
#undef SEQ6
#undef SEQ7
#undef SEQ8
#undef SEQ9

#undef NOP
#undef BRANCH
#undef REPEAT
#undef JMP
#undef GOTO

#undef RZ_IL_OPBUILDER_BEGIN_H
