#ifndef RZ_VERSION_H
#define RZ_VERSION_H 1
#define RZ_VERSION_MAJOR 0
#define RZ_VERSION_MINOR 4
#define RZ_VERSION_PATCH 1
#define RZ_VERSION_NUMBER 4001
#define RZ_VERSION "0.4.1"
#define RZ_GITTIP ""
#define RZ_BIRTH "2022-12-06__02:03:38"
/* #undef RZ_PACKAGER_VERSION */
/* #undef RZ_PACKAGER */
#endif
