// SPDX-FileCopyrightText: 2008-2016 pancake <pancake@nopcode.org>
// SPDX-FileCopyrightText: 2008-2016 r00tus3r <iamakshayajayan@gmail.com>
// SPDX-License-Identifier: LGPL-3.0-only

#ifndef RZ_ASCII_TABLE_H
#define RZ_ASCII_TABLE_H

#ifdef __cplusplus
extern "C" {
#endif

RZ_API const char *ret_ascii_table(void);

#ifdef __cplusplus
}
#endif

#endif
