// SPDX-FileCopyrightText: 2021 heersin <teablearcher@gmail.com>
// SPDX-License-Identifier: LGPL-3.0-only

#ifndef RZ_IL_DEFINITIONS_H
#define RZ_IL_DEFINITIONS_H

#include <rz_util/rz_bitvector.h>
#include <rz_il/definitions/label.h>
#include <rz_il/definitions/bool.h>
#include <rz_il/definitions/value.h>
#include <rz_il/definitions/variable.h>
#include <rz_il/definitions/mem.h>

#endif // RZ_IL_DEFINITIONS_H
